START TRANSACTION;

DELIMITER //

DROP PROCEDURE IF EXISTS GetDepartments //

# -------------------------------------------------------------------------------------------
# Label: faculties
# -------------------------------------------------------------------------------------------

# DROP FUNCTION IF EXISTS CreateFaculty //
DROP FUNCTION IF EXISTS Faculty_Create //
CREATE FUNCTION Faculty_Create (
  pFacultyExternalID TEXT CHARSET utf8,
  pFacultyName TEXT CHARSET utf8,
  pFacultyAbbr TEXT CHARSET utf8
) RETURNS INT(11) # -1 or id
NO SQL
BEGIN
  DECLARE vInnerID INT DEFAULT -1;
  DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;

  SET vInnerID = (SELECT ID
                  FROM faculties
                  WHERE faculties.ExternalID LIKE pFacultyExternalID OR faculties.Name LIKE pFacultyName
                  LIMIT 1);

  IF vInnerID IS NULL THEN
    INSERT INTO faculties (ExternalID, Name, Abbr) VALUES (pFacultyExternalID, pFacultyName, pFacultyAbbr);
    RETURN LAST_INSERT_ID();
  ELSE
    RETURN (SELECT Faculty_Update(vInnerID, pFacultyExternalID, pFacultyName, pFacultyAbbr));
  END IF;
END//

DROP FUNCTION IF EXISTS Faculty_Update //
CREATE FUNCTION Faculty_Update (
  pFacultyID int(11),
  pFacultyExternalID TEXT CHARSET utf8,
  pFacultyName TEXT CHARSET utf8,
  pFacultyAbbr TEXT CHARSET utf8
) RETURNS INT(11) # -1 or id
NO SQL
BEGIN
  DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;

  IF pFacultyExternalID IS NOT NULL THEN
    UPDATE faculties SET ExternalID = pFacultyExternalID WHERE ID = pFacultyID;
  END IF;

  IF pFacultyName IS NOT NULL THEN
    UPDATE faculties SET Name = pFacultyName WHERE ID = pFacultyID;
  END IF;

  IF pFacultyAbbr IS NOT NULL THEN
    UPDATE faculties SET Abbr = pFacultyAbbr WHERE ID = pFacultyID;
  END IF;

  RETURN pFacultyID;
END//

-- Получение внутреннего ID по коду справочника из 1С
DROP PROCEDURE IF EXISTS Faculty_GetIdByExternalID//
CREATE PROCEDURE Faculty_GetIdByExternalID(IN pFacultyExternalID VARCHAR(9))
READS SQL DATA
BEGIN
  SELECT faculties.ID as ID
  FROM faculties
  WHERE faculties.ExternalID = pFacultyExternalID;
END//

# -------------------------------------------------------------------------------------------
# Label: departments
# -------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS Department_GetIdByName//
CREATE PROCEDURE Department_GetIdByName(IN pDepartmentName varchar(200))
READS SQL DATA
BEGIN
  SELECT departments.ID as ID
  FROM departments
  WHERE departments.Name = pDepartmentName;
END//

DROP PROCEDURE IF EXISTS Department_GetIdByExternalID//
CREATE PROCEDURE Department_GetIdByExternalID(IN pDepartmentExternalID VARCHAR(9))
READS SQL DATA
BEGIN
  SELECT departments.ID as ID
  FROM departments
  WHERE departments.ExternalID = pDepartmentExternalID;
END//

DROP PROCEDURE IF EXISTS Departments_Get//
CREATE PROCEDURE Departments_Get (IN pFacultyID INT)
READS SQL DATA
BEGIN
  IF pFacultyID <=> 0 THEN
    SELECT departments.* FROM departments
    ORDER BY departments.Name ASC;
  ELSE
    SELECT  departments.* -- ID, Name, FacultyID
    FROM departments
    WHERE departments.FacultyID = pFacultyID
    ORDER BY departments.Name ASC;
  END IF;
END //

# create department or return existing
DROP FUNCTION IF EXISTS Department_Create //
CREATE FUNCTION Department_Create (
  pName VARCHAR(200) CHARSET utf8,
  pFacultyID INT(11)
) RETURNS INT(11) # department id or -1 if failed
NO SQL
BEGIN
  DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;

  INSERT INTO departments (Name, FacultyID) VALUES (pName, pFacultyID)
  ON DUPLICATE KEY UPDATE
    departments.ID = LAST_INSERT_ID(departments.ID);
  RETURN LAST_INSERT_ID();
END //

DROP FUNCTION IF EXISTS Department_Create_With_ExternalID //
CREATE FUNCTION Department_Create_With_ExternalID ( #:externalID, :name, :facultyExternalID
  pDepartmentExternalID TEXT CHARSET utf8,
  pDepartmentName TEXT CHARSET utf8,
  pFacultyExternalID TEXT CHARSET utf8
) RETURNS INT(11) # -1 or id
NO SQL
BEGIN
  DECLARE vInnerID, vFacultyInnerID INT DEFAULT -1;
  DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;

  SET vFacultyInnerID = Faculty_GetIdByExternalID(pFacultyExternalID);
  IF vFacultyInnerID IS NULL THEN
    RETURN -2;
  END IF;

  SET vInnerID = (SELECT ID
                  FROM departments
                  WHERE departments.ExternalID LIKE pDepartmentExternalID OR departments.Name LIKE pDepartmentName
                  LIMIT 1);

  IF vInnerID IS NULL THEN
    INSERT INTO departments (ExternalID, Name, FacultyID) VALUES (pDepartmentExternalID, pDepartmentName, vFacultyInnerID);
    RETURN LAST_INSERT_ID();
  ELSE
    RETURN (SELECT Department_Update(vInnerID, pDepartmentExternalID, pDepartmentName, vFacultyInnerID));
  END IF;
END//

DROP FUNCTION IF EXISTS Department_Update //
CREATE FUNCTION Department_Update (
  pDepartmentID int(11),
  pDepartmentExternalID TEXT CHARSET utf8,
  pDepartmentName TEXT CHARSET utf8,
  pFacultyID TEXT CHARSET utf8
) RETURNS INT(11) # -1 or id
NO SQL
BEGIN
  DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;

  IF pDepartmentExternalID IS NOT NULL THEN
    UPDATE departments SET ExternalID = pDepartmentExternalID WHERE ID = pDepartmentID;
  END IF;

  IF pDepartmentName IS NOT NULL THEN
    UPDATE departments SET Name = pDepartmentName WHERE ID = pDepartmentID;
  END IF;

  IF pFacultyID IS NOT NULL THEN
    UPDATE departments SET FacultyID = pFacultyID WHERE ID = pDepartmentID;
  END IF;

  RETURN pDepartmentID;
END//

DROP FUNCTION IF EXISTS Department_Search //
CREATE FUNCTION Department_Search (
  pName VARCHAR(200) CHARSET utf8,
  pFacultyID INT(11)
) RETURNS INT(11) # department id of -1 if not found
READS SQL DATA
  RETURN COALESCE((
                    SELECT departments.ID FROM departments
                    WHERE   departments.Name = pName AND departments.FacultyID = pFacultyID LIMIT 1
                  ), -1)//

# -------------------------------------------------------------------------------------------
# Label: job positions
# -------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS JobPositions_Get//
CREATE PROCEDURE JobPositions_Get ()
READS SQL DATA
  SELECT  job_positions.* -- ID, Name
  FROM    job_positions
  ORDER BY job_positions.Name //

DROP PROCEDURE IF EXISTS JobPosition_GetIdByName//
CREATE PROCEDURE JobPosition_GetIdByName(IN pJobPositionName varchar(200))
READS SQL DATA
BEGIN
  SELECT job_positions.ID as ID
  FROM job_positions
  WHERE job_positions.Name = pJobPositionName;
END//

DROP FUNCTION IF EXISTS JobPosition_Create //
CREATE FUNCTION JobPosition_Create (
  pJobPositionName varchar(200) CHARSET utf8
) RETURNS INT(11) # -1 or id
NO SQL
BEGIN
  DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;

  INSERT INTO job_positions (Name) VALUES (pJobPositionName);
  RETURN LAST_INSERT_ID();
END//

DELIMITER ;

COMMIT ;
